local T, C, L, _ = unpack(select(2, ...))
if C.announcements.flask_food ~= true then return end

----------------------------------------------------------------------------------------
--	Checks the usage of flasks and food of your party/raid members(ffCheck by Silverwind)
----------------------------------------------------------------------------------------
local noFood, noFlask, unitBuffs, noIntellect, noMark, noSpirit, noStamina, noKings = {}, {}, {}, {}, {}, {}, {}, {}
local foods = T.ReminderBuffs["Food"]
local flasks = T.ReminderBuffs["Flask"]
local Intellect = T.ReminderBuffs["Intellect"]
local Mark = T.ReminderBuffs["Mark"]
local Spirit = T.ReminderBuffs["Spirit"]
local Stamina = T.ReminderBuffs["Stamina"]
local Kings = T.ReminderBuffs["Kings"]

local function checkMana(unit)
	local _, powerTypeString = UnitPowerType(unit)
	if powerTypeString == "MANA" then
		return true
	end
end

local function scan(unit)
	table.wipe(unitBuffs)
	local i = 1
	while true do
		local name = UnitAura(unit, i, "HELPFUL")
		if not name then return end
		unitBuffs[name] = true
		i = i + 1
	end
end

local function checkFood()
	for i = 1, #foods do
		local name = unpack(foods[i])
		if unitBuffs[name] then
			return true
		end
	end
end

local function checkFlask()
	for i = 1, #flasks do
		local name = unpack(flasks[i])
		if unitBuffs[name] then
			return true
		end
	end
end

local function checkIntellect()
	for i = 1, #Intellect do
		local name = unpack(Intellect[i])
		if unitBuffs[name] then
			return true
		end
	end
end

local function checkMark()
	for i = 1, #Mark do
		local name = unpack(Mark[i])
		if unitBuffs[name] then
			return true
		end
	end
end

local function checkSpirit()
	for i = 1, #Spirit do
		local name = unpack(Spirit[i])
		if unitBuffs[name] then
			return true
		end
	end
end

local function checkStamina()
	for i = 1, #Stamina do
		local name = unpack(Stamina[i])
		if unitBuffs[name] then
			return true
		end
	end
end

local function checkKings()
	for i = 1, #Kings do
		local name = unpack(Kings[i])
		if unitBuffs[name] then
			return true
		end
	end
end

local function checkUnit(unit)
	local name = UnitName(unit)
	scan(unit)
	if not checkFood() then
		noFood[#noFood + 1] = name
	end
	if not checkFlask() then
		noFlask[#noFlask + 1] = name
	end
	if not checkIntellect() and checkMana(unit) then
		noIntellect[#noIntellect + 1] = name
	end
	if not checkMark() then
		noMark[#noMark + 1] = name
	end
	if not checkSpirit() and checkMana(unit) then
		noSpirit[#noSpirit + 1] = name
	end
	if not checkStamina() then
		noStamina[#noStamina + 1] = name
	end
	if T.faction == Alliance then		
		if not checkKings() then
			noKings[#noKings + 1] = name
		end
	end
end

local function print(text)
	_G.print("|cffffff00"..text.."|r")
end

-- The Main function to run a check
local function run()
	local checkType
	local output

	if C.announcements.flask_food_auto == true then C.announcements.flask_food_raid = true end

	table.wipe(noFood)
	table.wipe(noFlask)
	table.wipe(noIntellect)
	table.wipe(noMark)
	table.wipe(noSpirit)
	table.wipe(noStamina)
	table.wipe(noKings)
	
	if UnitInRaid("player") then
		checkType = "raid"
	else
		checkType = "party"
		checkUnit("player")
	end

	for i = 1, GetNumGroupMembers() do
		if checkType == "raid" then
			local online = select(8, GetRaidRosterInfo(i))
			if online then
				local unit = checkType..i
				checkUnit(unit)
			end
		else
			local unit = checkType..i
			if UnitIsConnected(unit) then
				checkUnit(unit)
			end
		end
	end

	if #noFlask > 0 then
		table.sort(noFlask)
		if #noFlask > 8 then
		    output = L_ANNOUNCE_FF_NOFLASK..format(WHO_NUM_RESULTS, #noFlask)
		else
			output = L_ANNOUNCE_FF_NOFLASK..table.concat(noFlask, ", ")
		end
		if C.announcements.flask_food_raid then
			SendChatMessage(output, T.CheckChat())
		else
			print(output)
		end
	end

	if #noFood > 0 then
		table.sort(noFood)
		if #noFood > 8 then
 			output = L_ANNOUNCE_FF_NOFOOD..format(WHO_NUM_RESULTS, #noFood)
 		else	
			output = L_ANNOUNCE_FF_NOFOOD..table.concat(noFood, ", ")
		end
		if C.announcements.flask_food_raid then
			SendChatMessage(output, T.CheckChat())
		else
			print(output)
		end
	end

	if #noIntellect > 0 then
		table.sort(noIntellect)
		if #noIntellect > 8 then
		    output = L_ANNOUNCE_FF_NOIntellect..format(WHO_NUM_RESULTS, #noIntellect)
		else
			output = L_ANNOUNCE_FF_NOIntellect..table.concat(noIntellect, ", ")
		end
		if C.announcements.flask_food_raid then
			SendChatMessage(output, T.CheckChat())
		else
			print(output)
		end
	end

	if #noMark > 0 then
		table.sort(noMark)
		if #noMark > 8 then
		    output = L_ANNOUNCE_FF_NOMark..format(WHO_NUM_RESULTS, #noMark)
		else
			output = L_ANNOUNCE_FF_NOMark..table.concat(noMark, ", ")
		end
		if C.announcements.flask_food_raid then
			SendChatMessage(output, T.CheckChat())
		else
			print(output)
		end
	end

	if #noSpirit > 0 then
		table.sort(noSpirit)
		if #noSpirit > 8 then
		    output = L_ANNOUNCE_FF_NOSpirit..format(WHO_NUM_RESULTS, #noSpirit)
		else
			output = L_ANNOUNCE_FF_NOSpirit..table.concat(noSpirit, ", ")
		end
		if C.announcements.flask_food_raid then
			SendChatMessage(output, T.CheckChat())
		else
			print(output)
		end
	end

	if #noStamina > 0 then
		table.sort(noStamina)
		if #noStamina > 8 then
		    output = L_ANNOUNCE_FF_NOStamina..format(WHO_NUM_RESULTS, #noStamina)
		else
			output = L_ANNOUNCE_FF_NOStamina..table.concat(noStamina, ", ")
		end
		if C.announcements.flask_food_raid then
			SendChatMessage(output, T.CheckChat())
		else
			print(output)
		end
	end

	if #noKings > 0 then
		table.sort(noKings)
		if #noKings > 8 then
		    output = L_ANNOUNCE_FF_NOKings..format(WHO_NUM_RESULTS, #noKings)
		else
			output = L_ANNOUNCE_FF_NOKings..table.concat(noKings, ", ")
		end
		if C.announcements.flask_food_raid then
			SendChatMessage(output, T.CheckChat())
		else
			print(output)
		end
	end

	if #noFood == 0 and #noFlask == 0 and #noKings == 0 and #noIntellect == 0 and #noMark == 0 and #noSpirit == 0 and #noStamina == 0 then
		if C.announcements.flask_food_raid then
			SendChatMessage(L_ANNOUNCE_FF_ALLBUFFED, T.CheckChat())
		else
			print(L_ANNOUNCE_FF_ALLBUFFED)
		end
	end
end

-- Event Handler
local frame = CreateFrame("Frame")
frame:RegisterEvent("READY_CHECK")
frame:SetScript("OnEvent", function()
	if C.announcements.flask_food_auto then
		run()
	end
end)

-- Slash command
SlashCmdList.FFCHECK = run
SLASH_FFCHECK1 = "/ffcheck"
SLASH_FFCHECK2 = "/аасрусл"
SLASH_FFCHECK3 = "/fc"

-- Check button
if C.misc.raid_tools == true then
	RaidUtilityPanel:SetHeight(168)

	local button = CreateFrame("Button", "FoodFlaskCheckButton", RaidUtilityPanel, "UIPanelButtonTemplate")
	button:SetWidth(RaidUtilityPanel:GetWidth() * 0.8)
	button:SetHeight(18)
	button:SetPoint("TOP", RaidUtilityRaidControlButton, "BOTTOM", 0, -5)
	if IsAddOnLoaded("Aurora") then
		local F = unpack(Aurora)
		F.Reskin(button)
	else
		button:SkinButton()
	end
	button:EnableMouse(true)
	button:SetScript("OnMouseUp", function(self) run() end)

	local t = button:CreateFontString(nil, "OVERLAY", button)
	t:SetFont(C.media.pixel_font, C.media.pixel_font_size, C.media.pixel_font_style)
	t:SetPoint("CENTER")
	t:SetJustifyH("CENTER")
	t:SetText(L_ANNOUNCE_FF_CHECK_BUTTON)
end
